class Route{
  Station sStation, eStation;
  float[] duration;
  int[] nTrips;
  boolean isGroup;
  boolean isLocal;
  boolean toDisplay;
  float hue;
  int opac;
  float satmap, brimap;
  String acct;
  
  Route(Station sStation, Station eStation, String acct){
    this.sStation = sStation;  // start station
    this.eStation = eStation;  // end station
    this.duration = new float[nMonths];  // avg ride durations by month
    this.nTrips = new int[nMonths];      // no. rides by month
    for (int t=0; t<nMonths; t++) this.duration[t] = 0;
    for (TableRow row : durationTable.matchRows(str(sStation.sNumber),"start_station_number")) {
      if (row.getInt("end_station_number")==eStation.sNumber & 
          row.getString("account_type").equals(acct)) {
        duration[row.getInt("month")-1] = row.getFloat("avg(duration)");
        nTrips[row.getInt("month")-1] = row.getInt("ridesBetween");
      }
    }
    if (acct.equals("Registered")) {
          this.isLocal = true;
          this.hue = 17;
        }
    if (acct.equals("Casual")) {
          this.isLocal = false;
          this.hue = 200;
    }
    this.opac = 50;
    this.toDisplay = true;
    if (max1D(duration)==0) this.toDisplay = false;
  }
  
  void drawRoute() {
    
    if (toDisplay){
    
    // define coordinate transformation variables
    // new translated coordinates of start station
    float sXnew = (sStation.stationX - eStation.stationX) / 2;
    float sYnew = (sStation.stationY - eStation.stationY) / 2;
    float sZnew = (sStation.stationZ - eStation.stationZ) / 2;
    // spherical coordinates of start station (n-azimuth from bottom plane)
    float rho = sqrt( sq(sXnew) + sq(sYnew) + sq(sZnew) );
    float polarR = atan(-1*sXnew/sZnew);
    if (sZnew==0) polarR = atan(-1*sXnew/(sZnew+0.000001));
    float nazimR = PI/2 - atan(-1 * sqrt( sq(sXnew) + sq(sZnew) ) / sYnew);
    
    // start coordinate transformation
    pushMatrix();
    // translate to new origin halfway between stations
    translate((sStation.stationX + eStation.stationX) /2,
              (sStation.stationY + eStation.stationY) /2,
              (sStation.stationZ + eStation.stationZ) /2);
    // rotate through polar angle in spherical coords          
    rotateY(-1 * polarR);
    // rotate up through pi/2 - azimuth in spherical coords
    if (sZnew >= 0) rotateX(nazimR);
    else if (sZnew < 0) rotateX(-1 * nazimR);
       
    // finish draw of route
    this.drawRouteCurve(rho);
    popMatrix();
    }
  }
    
  // line draw function
  void drawRouteCurve(float halfDist) { 
    float ptSep = 2 * halfDist / (nMonths+1);
    float maxD = 0;
    if (lType==2) maxD = max1D(duration);
    if (lType==1) maxD = max1D(float(nTrips));
    beginShape();
    
    // first anchor point
    satmap = map(0,0,nMonths+1,satMin,satMax);
    brimap = map(0,0,maxD,briMin,briMax);
    stroke(hue,satmap,brimap,opac);
    strokeWeight(2);
    noFill();
    curveVertex(0,0,halfDist);
    
    // point for each trip
    for (int d=0; d<nMonths+2; d++) {  // loop over trips      
      // draw a route curve
          float z = halfDist - ptSep*d;
          float x = 0;
          float y = 0;
          if (d!=0 & d!=nMonths+1) {
            if (lType==1) y = (duration[d-1]*dscale+1);
            if (lType==2) y = (nTrips[d-1]*tscale+1);
          }
          satmap = map(d,0,nMonths+1,satMin,satMax);
          brimap = map(y,0,maxD,briMin,briMax);
          stroke(hue,satmap,brimap,opac);
          strokeWeight(2);
          noFill();
          curveVertex(x, y, z);
      }
      
      // last anchor point
      satmap = map(nMonths+1,0,nMonths+1,satMin,satMax);
      brimap = map(1,0,maxD,briMin,briMax);
      stroke(hue,satmap,brimap,opac);
      strokeWeight(2);
      noFill();
      curveVertex(0,0,-1*halfDist);
      
      endShape(OPEN);
  }
  
  void setOpac(boolean isMouse) {
    // sets opacity of route based on mouseover status
    if (isMouse==true) this.opac = 200;
    else if (isMouse==false) this.opac = 50;
  }
  
  void setVis(boolean isVis) {
    // sets visibility of route
    if (isVis==false) this.toDisplay = false;
    else if (isVis==true & max1D(this.duration)>0) this.toDisplay = true; 
  }

}