/*************************************************************************************
3D Bikeshare Data Visualization
                                                     
Author: Kimberly Schlesinger                        
Supervisor: George Legrady                           
*************************************************************************************/

// declare camera
import peasy.*;
PeasyCam cam;

// general network organization
int nStations;
int nRoutes;
ArrayList<Route> Routes = new ArrayList<Route>();
ArrayList<Station> Stations = new ArrayList<Station>();
ArrayList<Station> mStations = new ArrayList<Station>();
IntDict stationIndex; // saves station indices in Stations ArrayList indexed by sNumber

// layout settings
float dist = 15; // xz-distance scale between 2 stations
float ht = 3;    // y-distance (height) between stations
float dscale = 0.02; // make smaller to make duration data smaller
float tscale = 2;    // make smaller to make nTrips data smaller
int nMonths = 9;     // number of months to display
float lM = 80;  // text alignment (left margin)
float tS = 20;  // text alignment (line spacing)      

// tables
Table stationTable;  // all stations, names, and total trips
Table nTripsTable;   // nTrips between each station pair
Table sLocTable;     // station locations
Table durationTable; // avg duration and nTrips per month/route
Table yTable;        // graph metrics for y-coordinates

// min and max values for route
// saturation and brightness
float satMin = 25; //35
float satMax = 99; //99
float briMin = 75; //75
float briMax = 99; //99

// for labels/legends
String theTxt;
float[] random1; 
float[] random2;

// interaction switches
boolean iSwitch;  // interface
boolean dSwitch;  // degree/centrality
boolean mSwitch;  // mouseover label
boolean dispTxt;  // text to display
boolean vSwitch;  // route visibility
boolean hSwitch;  // route highlighting

void setup(){
  // size and basic display modes
  size(1200,750,P3D);
  colorMode(HSB, 360, 100, 100);
  translate(width/2,height/2,0);
  surface.setResizable(true);
  
  setupCamera();

  setSwitches();
  
  loadTables();
    
  createStations();
  
  createRoutes();

}

void draw(){
 
  // set background and initial position of array
  background(0,0,10);
  dispTxt = false;     // reset mouseover
  theTxt = "";         // reset mouseover
  mStations.clear();   // reset mouseover
  cam.rotateX(0);
    
  // draw stations
  for (int s=0; s<Stations.size(); s++){
    Stations.get(s).drawStation();
  }
  
  // draw routes
  for (int r=0; r<Routes.size(); r++){ 
    Route rt = Routes.get(r);
    if (aType==0 || (aType==1 & rt.isLocal==true) || (aType==2 & rt.isLocal==false)) {
      boolean toShow = (mStations.contains(rt.eStation) | mStations.contains(rt.sStation));
      rt.setOpac(toShow);
      if (vSwitch==false) rt.setVis(toShow);
      rt.drawRoute();
    }
  }
   
  // draw interface
  if (iSwitch==true) iDisplay();
  
  // draw mouseover text
  if (mSwitch==true) mDisplay();
   
}