# This implements the force-directed layout algorithm in 2D using networkx
# package. Imports a graph with weighted edges from .csv and outputs a set
# of 2D coordinates at which to plot the nodes.

import networkx as nx
import pandas as pd

# load in bike network from tripsBetween
rf = 'tripsBetween_DC2016.csv'
wf = 'stationLoc_DC.csv'
df = pd.read_csv(rf,sep=',')
df.rename(columns={'ridesBetween' : 'weight'}, inplace=True)
bikeG = nx.from_pandas_dataframe(df,'start_station_number','end_station_number',edge_attr='weight',create_using=nx.DiGraph())

# use networkx to create force-directed positions
pos = nx.nx_pydot.graphviz_layout(bikeG,prog='neato')#dim=2,k=100,weight='ridesBetween',scale=100,center=[0,0])

# save positions in proper format for plotting in processing
dfnew = pd.DataFrame.from_dict(pos,orient='index')

# center on x, y of the first node
dfnew[0] = dfnew[0] - dfnew.get_value(0,0,True)
dfnew[1] = dfnew[1] - dfnew.get_value(0,1,True)

# save dataframe as .csv with headers
dfnew.to_csv(wf,header=['x_value','y_value'],index=True,index_label='station_number')

