
// setup camera
void setupCamera() {
  cam = new PeasyCam(this, 1000);
  cam.setWheelScale(0.1);
  cam.setRotations(2.005,-0.895,1.898);
  cam.lookAt(-798.0758, 191.97868, 1952.2181);
  cam.setDistance(5000);
  cam.setMinimumDistance(50);
  cam.setMaximumDistance(500000);
}

void setSwitches() {
  // set all interaction switches to defaults
  iSwitch = true;              // instructions on
  dSwitch = true;              // degrees on
  mSwitch = true;              // mouseover text on
  dispTxt = false;             // no initial mouseover txt
  hSwitch = true;              // mouseover highlighting enabled
  vSwitch = true;              // all routes visible
}

void loadTables() {
  // load in data to tables
  stationTable = loadTable("dc_stations.csv","header");
  nTripsTable = loadTable("200tripsBetween_DC2016.csv","header");
  //nTripsTable = loadTable("tripsBetween_DC2016.csv","header");
  sLocTable = loadTable("stationLoc_DC.csv","header");  
  //durationTable = loadTable("avgSecDuration_ridesBetween_acctType_byMonth_2016.csv","header");
  durationTable = loadTable("avgSecDuration_200ridesBetween_acctType_byMonth_2016.csv","header");
  yTable = loadTable("y_metrics_2016.csv","header");
  
  // initialize random trajectories for legend
  random1 = new float[10];
  random2 = new float[10];
  for (int i=0; i<10;i++) {
    random1[i] = random(-10,10);
    random2[i] = random(-10,10);
  }
}

void createStations() {
  // set params
  stationIndex = new IntDict();
  nStations = stationTable.getRowCount();
 
 // create each station in a for loop
 for (int s=0; s<nStations; s++) {
   int sNum = stationTable.getInt(s,"station_number");
   Stations.add(new Station(sNum));
   stationIndex.set(str(sNum),s);
 }  // end loop over stations
}

void createRoutes() {
  // create each route in a for loop
  int nRows = nTripsTable.getRowCount();
  //int nRows = 10000;
  int nTripsShown = 0;
  println(nTripsTable.getRowCount());
  for (int r=0; r<nRows; r++) {
    if ( nTripsTable.getInt(r,"start_station_number")!=  // check stations not the same
         nTripsTable.getInt(r,"end_station_number") ) {
           //if (nTripsTable.getInt(r,"ridesBetween") > 200) { // check 'ridesBetween' > 0
             // look up relevant stations by ArrayList index and create route
             String sKey = str(nTripsTable.getInt(r,"start_station_number"));
             String eKey = str(nTripsTable.getInt(r,"end_station_number"));
            if (stationIndex.hasKey(sKey) & stationIndex.hasKey(eKey)) { 
             Station sSta = Stations.get(stationIndex.get(sKey));
             Station eSta = Stations.get(stationIndex.get(eKey));
             Routes.add(new Route(sSta,eSta,"Registered"));
             Routes.add(new Route(sSta,eSta,"Casual"));
             println("creating route " + r + "/" + nRows);
             nTripsShown = nTripsShown + 1;
            }
           //}
    }  // end if
  }  // end loop over r
  println(nTripsShown);
} // end createRoutes()